require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xdnsserveradzone',
  dscmeta_resource_friendly_name: 'xDnsServerADZone',
  dscmeta_resource_name: 'MSFT_xDnsServerADZone',
  dscmeta_module_name: 'xDnsServer',
  dscmeta_module_version: '1.10.0.0',
  docs: 'The DSC xDnsServerADZone resource type.
         Automatically generated from version 1.10.0.0',
  features: ['simple_get_filter', 'canonicalize'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    dsc_name: {
      type: 'String',
      desc: 'DNS Server zone name',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_computername: {
      type: 'Optional[String]',
      desc: 'DNS Server name',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_dynamicupdate: {
      type: "Optional[Enum['None', 'NonsecureAndSecure', 'Secure']]",
      desc: 'Dynamic zone update option',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_directorypartitionname: {
      type: 'Optional[String]',
      desc: 'Directory partition name',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_credential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Credential used to set zone',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_replicationscope: {
      type: "Enum['Custom', 'Domain', 'Forest', 'Legacy']",
      desc: 'Replication scope option',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: 'Whether the DNS zone should be available or removed',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
